@@ -8,6 +8,7 @@ import sys
 import time
 import tempfile
 import shutil
+import subprocess
 
 import mozprocess
 from mozprofile import FirefoxProfile, Preferences
@@ -34,6 +35,9 @@ def get_free_port(start_port, exclude=None):
         finally:
             s.close()
 
+class BrowserError(Exception):
+    pass
+
 class ProcessHandler(mozprocess.ProcessHandlerMixin):
     pass
 
@@ -163,6 +167,8 @@ class B2GBrowser(Browser):
 
     def __init__(self, logger, prefs_root):
         Browser.__init__(self, logger)
+        logger.info("Waiting for device")
+        subprocess.call(["adb", "wait-for-device"])
         self.device = mozdevice.DeviceManagerADB()
         self.marionette_port = get_free_port(2828, exclude=self.used_ports)
         self.used_ports.add(self.marionette_port)
@@ -210,6 +216,24 @@ class B2GBrowser(Browser):
 
         return preferences
 
+    def wait_for_net(self):
+        # TODO: limit how long we wait before we fail
+        # consider the possibility that wlan0 is not the right interface
+
+        self.logger.info("Waiting for net connection")
+        def has_connection():
+            try:
+                return self.device.getIP(["wlan0"]) is not None
+            except mozdevice.DMError:
+                return False
+
+        t0 = time.time()
+        timeout = 60
+        while not has_connection():
+            if time.time() - t0 > timeout:
+                raise BrowserError("Waiting for net timed out")
+            time.sleep(1)
+
     def stop(self):
         pass
 
@@ -222,7 +246,7 @@ class B2GBrowser(Browser):
             self.device.removeDir(remote)
             self.device.pushDir(local, remote)
         shutil.rmtree(self.backup_path)
-        self.device.reboot()
+        self.device.reboot(wait=True)
 
     def pid(self):
         return "Remote"
@@ -236,6 +260,7 @@ class B2GBrowser(Browser):
         executor.logger.debug("Running browser.after_connect steps")
         self.install_cert_app(executor)
         self.use_cert_app(executor)
+        self.wait_for_net()
 
     def install_cert_app(self, executor):
         marionette = executor.marionette
@@ -278,11 +303,21 @@ class B2GBrowser(Browser):
     def wait_for_homescreen(self, marionette):
         marionette.set_context(marionette.CONTEXT_CONTENT)
         marionette.execute_async_script("""
-log('waiting for mozbrowserloadend');
-window.addEventListener('mozbrowserloadend', function loaded(aEvent) {
-  log('received mozbrowserloadend for ' + aEvent.target.src);
-  if (aEvent.target.src.indexOf('ftu') != -1 || aEvent.target.src.indexOf('homescreen') != -1) {
-    window.removeEventListener('mozbrowserloadend', loaded);
-    marionetteScriptFinished();
-  }
-});""", script_timeout=30000)
+let manager = window.wrappedJSObject.AppWindowManager || window.wrappedJSObject.WindowManager;
+let app = ('getActiveApp' in manager) ? manager.getActiveApp() : manager.getCurrentDisplayedApp();
+log(app);
+if (app) {
+  log('Already loaded home screen');
+  marionetteScriptFinished();
+} else {
+  log('waiting for mozbrowserloadend');
+  window.addEventListener('mozbrowserloadend', function loaded(aEvent) {
+    log('received mozbrowserloadend for ' + aEvent.target.src);
+    if (aEvent.target.src.indexOf('ftu') != -1 || aEvent.target.src.indexOf('homescreen') != -1) {
+      window.removeEventListener('mozbrowserloadend', loaded);
+      let app = ('getActiveApp' in manager) ? manager.getActiveApp() : manager.getCurrentDisplayedApp();
+      log(app);
+      marionetteScriptFinished();
+    }
+  });
+}""", script_timeout=30000)
