@@ -281,6 +281,9 @@ class TestRunnerManager(threading.Thread):
         #It seems that this lock is helpful to prevent some race that otherwise
         #sometimes stops the spawned processes initalising correctly, and
         #leaves this thread hung
+        if self.init_timer is not None:
+            self.init_timer.cancel()
+
         self.logger.debug("Init called, starting browser and runner")
 
         def init_failed():
@@ -298,10 +301,15 @@ class TestRunnerManager(threading.Thread):
             #whatever reason
             #TODO: make this timeout configurable
             self.init_timer = threading.Timer(self.browser.init_timeout, init_failed)
-            self.init_timer.start()
+            try:
+                self.init_timer.start()
 
-            self.browser.start()
-            self.start_test_runner()
+                self.browser.start()
+                self.start_test_runner()
+            except:
+                self.init_timer.cancel()
+                self.logger.error(traceback.format_exc())
+                self.init_failed()
 
     def init_succeeded(self):
         """Callback when we have started the browser, connected via
@@ -342,6 +350,8 @@ class TestRunnerManager(threading.Thread):
         self.remote_queue.put((command, args))
 
     def cleanup(self):
+        if self.init_timer is not None:
+            self.init_timer.cancel()
         self.logger.debug("TestManager cleanup")
         while True:
             try:
@@ -358,8 +368,6 @@ class TestRunnerManager(threading.Thread):
     def teardown(self):
         self.logger.debug("teardown in testrunnermanager")
         self.test_runner_proc = None
-        # self.command_queue.cancel_join_thread()
-        # self.remote_queue.cancel_join_thread()
         self.command_queue.close()
         self.remote_queue.close()
         self.command_queue = None
