@@ -29,6 +29,7 @@ import metadata
 import manifestexpected
 import wpttest
 import wptcommandline
+import manifestinclude
 
 here = os.path.split(__file__)[0]
 
@@ -235,33 +236,39 @@ class EqualTimeChunker(TestChunker):
         for item in tests:
             yield item
 
-class ManifestFilter(object):
-    def __init__(self, include=None, exclude=None):
-        self.include = include if include is not None else []
-        self.exclude = exclude if exclude is not None else []
+class TestFilter(object):
+    def __init__(self, include=None, exclude=None, manifest_path=None):
+        if manifest_path is not None:
+            self.manifest = manifestinclude.get_manifest(manifest_path)
+        else:
+            self.manifest = manifestinclude.IncludeManifest.create()
+
+        if include or exclude:
+            self.manifest.set("skip", "true")
+
+        if include is not None:
+            for item in include:
+                self.manifest.add_include(item)
+
+        if exclude is not None:
+            for item in exclude:
+                self.manifest.add_exclude(item)
 
     def __call__(self, manifest_iter):
         for test_path, tests in manifest_iter:
             include_tests = set()
             for test in tests:
-                if self.include:
-                    include = any(test.url.startswith(inc) for inc in self.include)
-                else:
-                    include = True
-                if self.exclude:
-                    exclude = any(test.url.startswith(exc) for exc in self.exclude)
-                else:
-                    exclude = False
-                if include and not exclude:
+                if self.manifest.include(test):
                     include_tests.add(test)
 
             if include_tests:
                 yield test_path, include_tests
 
 class TestLoader(object):
-    def __init__(self, tests_root, metadata_root, run_info):
+    def __init__(self, tests_root, metadata_root, test_filter, run_info):
         self.tests_root = tests_root
         self.metadata_root = metadata_root
+        self.test_filter = test_filter
         self.run_info = run_info
         self.manifest = self.load_manifest()
         self.tests = None
@@ -280,12 +287,11 @@ class TestLoader(object):
     def load_expected_manifest(self, test_path):
         return manifestexpected.get_manifest(self.metadata_root, test_path, self.run_info)
 
-    def load_tests(self, test_types, include_filters, chunk_type, total_chunks, chunk_number):
+    def load_tests(self, test_types, chunk_type, total_chunks, chunk_number):
         """Read in the tests from the manifest file and add them to a queue"""
         rv = defaultdict(list)
 
-        manifest_filter = ManifestFilter(include=include_filters)
-        manifest_items = manifest_filter(self.manifest.itertypes(*test_types))
+        manifest_items = self.test_filter(self.manifest.itertypes(*test_types))
 
         chunker = {"none": Unchunked,
                    "hash": HashChunker,
@@ -302,9 +308,9 @@ class TestLoader(object):
 
         return rv
 
-    def get_groups(self, test_types, include_filters=None, chunk_type="none", total_chunks=1, chunk_number=1):
+    def get_groups(self, test_types, chunk_type="none", total_chunks=1, chunk_number=1):
         if self.tests is None:
-            self.tests = self.load_tests(test_types, include_filters, chunk_type, total_chunks, chunk_number)
+            self.tests = self.load_tests(test_types, chunk_type, total_chunks, chunk_number)
 
         groups = set()
 
@@ -315,9 +321,9 @@ class TestLoader(object):
 
         return groups
 
-    def queue_tests(self, test_types, include_filters, chunk_type, total_chunks, chunk_number):
+    def queue_tests(self, test_types, chunk_type, total_chunks, chunk_number):
         if self.tests is None:
-            self.tests = self.load_tests(test_types, include_filters, chunk_type, total_chunks, chunk_number)
+            self.tests = self.load_tests(test_types, chunk_type, total_chunks, chunk_number)
 
         tests_queue = defaultdict(Queue)
         test_ids = []
@@ -334,6 +340,7 @@ class LogThread(threading.Thread):
         self.queue = queue
         self.log_func = getattr(logger, level)
         threading.Thread.__init__(self, name="Thread-Log")
+        self.daemon = True
 
     def run(self):
         while True:
@@ -407,14 +414,15 @@ def get_executor(product, test_type, http_server_url, timeout_multiplier):
 
 def list_test_groups(tests_root, metadata_root, test_types, product, **kwargs):
     run_info = wpttest.get_run_info(product, debug=False)
-    test_loader = TestLoader(tests_root, metadata_root, run_info)
+    test_loader = TestLoader(tests_root, metadata_root, TestFilter(), run_info)
 
     for item in sorted(test_loader.get_groups(test_types)):
         print item
 
 def run_tests(tests_root, metadata_root, prefs_root, test_types, binary=None,
-              processes=1, include=None, capture_stdio=True, product="firefox",
-              chunk_type="none", total_chunks=1, this_chunk=1, timeout_multiplier=1,
+              processes=1, include=None, exclude=None, include_manifest=None,
+              capture_stdio=True, product="firefox", chunk_type="none",
+              total_chunks=1, this_chunk=1, timeout_multiplier=1,
               repeat=1, **kwargs):
     logging_queue = None
     original_stdio = (sys.stdout, sys.stderr)
@@ -442,7 +450,9 @@ def run_tests(tests_root, metadata_root, prefs_root, test_types, binary=None,
         if "test_loader" in kwargs:
             test_loader = kwargs["test_loader"]
         else:
-            test_loader = TestLoader(tests_root, metadata_root, run_info)
+            test_filter = TestFilter(include=include, exclude=exclude,
+                                     manifest_path=include_manifest)
+            test_loader = TestLoader(tests_root, metadata_root, test_filter, run_info)
 
         with TestEnvironment(tests_root, env_options) as test_environment:
             base_server = "http://%s:%i" % (test_environment.config["host"],
@@ -450,7 +460,7 @@ def run_tests(tests_root, metadata_root, prefs_root, test_types, binary=None,
             for repeat_count in xrange(repeat):
                 if repeat > 1:
                     logger.info("Repetition %i / %i" % (repeat_count + 1, repeat))
-                test_ids, test_queues = test_loader.queue_tests(test_types, include, chunk_type, total_chunks, this_chunk)
+                test_ids, test_queues = test_loader.queue_tests(test_types, chunk_type, total_chunks, this_chunk)
                 unexpected_count = 0
                 logger.suite_start(test_ids, run_info)
                 for test_type in test_types:
